/*	License
 * 		inside EU: EUPL
 * 		outside EU: GPL or EUPL
 *  The program comes AS IS and without any warranty unless required by law
 *  (C) 2012 thecodemonkey
 */

package gui.filetree;

import java.io.IOException;
import java.nio.file.DirectoryStream;
import java.nio.file.FileSystems;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.DirectoryStream.Filter;
import java.nio.file.Paths;
import java.util.Collections;
import java.util.LinkedList;

import javax.swing.BorderFactory;
import javax.swing.JScrollPane;
import javax.swing.JTree;
import javax.swing.event.TreeExpansionEvent;
import javax.swing.event.TreeExpansionListener;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreePath;

import gui.filelist.FileList;

public class FileTree extends JScrollPane {

	private static final long serialVersionUID = -5417751980852395150L;

	private FileList fileList;
	private JTree fileTree;
	private DefaultTreeModel fileTreeModel;
	private FileTreeNode root;

	public FileTree() {
		this(null);
	}

	public FileTree(FileList fileList) {
		this.fileList = fileList;
		loadFileSystem();
		// fileTree.setCellRenderer(new TreeCellRenderer() {
		//
		// @Override
		// public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
		// return new JLabel(value.toString(), getIcon( ((FileTreeNode)value).getFile()),JLabel.HORIZONTAL);
		// }
		// });
	}

	public void expandFileTree(final Path dir) {
		if (dir != null) {		
				
			
				FileTreeNode curr = root;
				TreePath tp = null;

				if (curr.getFile() == null) {
					for (int j = 0; j < curr.getChildCount(); j++) {
						FileTreeNode n = (FileTreeNode) curr.getChildAt(j);
						if (n.getFile().equals(dir.getRoot())) {
							curr = n;
							tp = new TreePath(n.getPath());
							n.removeAllChildren();
							loadSubDirs(n);
//							fileTreeModel.nodeStructureChanged(n);
							fileTree.expandPath(tp);
							break;
						}
					}
				}

				if (dir.getNameCount() > 0) {
					tp = null;
					for (int i = 0; i < dir.getNameCount(); i++) {
						for (int j = 0; j < curr.getChildCount(); j++) {
							FileTreeNode n = (FileTreeNode) curr.getChildAt(j);
							Path p = n.getFile();
							if (p!=null)p=p.getFileName();
							if (p != null && p.equals(dir.getName(i))) {
								curr = n;
								tp = new TreePath(n.getPath());
								n.removeAllChildren();
								loadSubDirs(n);
//								fileTreeModel.nodeStructureChanged(n);
								fileTree.expandPath(tp);
								break;
							}
						}
					}
				
		
		
				if (tp != null)	{
					fileTree.expandPath(tp);
					fileTree.setSelectionPath(tp);
					fileTree.scrollPathToVisible(tp);
				}
				fileTree.repaint();
			}
			
		} 
	}

	public Path getSelectedDir() {
		if (fileTree.getSelectionPath() != null)
			return ((FileTreeNode) fileTree.getSelectionPath().getLastPathComponent()).getFile();
		else
			return null;
	}

	private void loadFileSystem() {
		Iterable<Path> roots = FileSystems.getDefault().getRootDirectories();

		LinkedList<Path> rootsSorted = new LinkedList<>();

		for (Path p : roots) {
			rootsSorted.add(p);
		}
		Collections.sort(rootsSorted);

		if (rootsSorted.size() == 1) {
			Path p = rootsSorted.getFirst();
			root = new FileTreeNode(p, p.toString());
			loadSubDirs(root);
		} else {
			root = new FileTreeNode();
			root.setDisplayName("File system");

			for (Path p : rootsSorted) {
				FileTreeNode n = new FileTreeNode(p, p.toString());
				n.add(FileTreeNode.getEmptyNode());
				root.add(n);
			}
			Path p = Paths.get(System.getProperty("user.home")).getParent();
			FileTreeNode n = new FileTreeNode(p, p.getFileName().toString());
			n.add(FileTreeNode.getEmptyNode());
			root.add(n);
		}

		this.fileTreeModel = new DefaultTreeModel(root);
		this.fileTree = new JTree(fileTreeModel);

		fileTree.setCellRenderer(new FileTreeCellRenderer());
		fileTree.setBorder(BorderFactory.createEmptyBorder(0, 8, 0, 0));
		setViewportView(fileTree);
		
		if (fileList!=null){
			fileTree.getSelectionModel().addTreeSelectionListener(new TreeSelectionListener() {
	
				@Override
				public void valueChanged(TreeSelectionEvent e) {
					TreePath p = (TreePath) e.getPath();
					if (p != null) {
	
						FileTreeNode n = (FileTreeNode) p.getLastPathComponent();
	
						if (n != null && n.getFile() != null) {
							try {
								fileList.loadDir(n.getFile());
							} catch (IOException e1) {
								e1.printStackTrace();
							}
						} else {
							fileList.clear();
						}
					}
				}
			});
		}
		fileTree.addTreeExpansionListener(new TreeExpansionListener() {

			@Override
			public void treeExpanded(TreeExpansionEvent event) {
				final TreePath p = event.getPath();
				final FileTreeNode n = (FileTreeNode) p.getLastPathComponent();

				if (n.getFile() != null) {
					n.removeAllChildren();
					loadSubDirs(n);
					fileTreeModel.nodeStructureChanged(n);
				} else if (n!=root){
					n.add(FileTreeNode.getEmptyNode());
				}
			}

			@Override
			public void treeCollapsed(TreeExpansionEvent event) {

			}
		});
	}

	private void loadSubDirs(FileTreeNode n) {
		if (Files.isReadable(n.getFile())) {
			try {

				DirectoryStream<Path> stream = Files.newDirectoryStream(n.getFile(), new Filter<Path>() {

					@Override
					public boolean accept(Path f) {
						try {
							return Files.isDirectory(f) && !Files.isHidden(f);
						} catch (IOException e) {
							return false;
						}
					}
				});
				LinkedList<Path> sorted = new LinkedList<>();
				for (Path p : stream) {
					sorted.add(p);
				}
				if (sorted.isEmpty()) {
					n.add(FileTreeNode.getEmptyNode());
				} else {
					Collections.sort(sorted);
					for (Path p : sorted) {
						FileTreeNode node = new FileTreeNode(p, p.getFileName().toString());
						node.add(FileTreeNode.getEmptyNode());
						n.add(node);
					}
				}
			} catch (IOException e) {
				e.printStackTrace();
			}
		} else {
			n.add(FileTreeNode.getEmptyNode());
		}
	}

}
