package gui.util.tablemodel;

import java.util.Vector;

import javax.swing.table.DefaultTableModel;

public class ObjectTableModel<V> extends DefaultTableModel {

	private static final long serialVersionUID = 7367032305913199269L;

	private Vector<V> vals;
	
	public ObjectTableModel(String...columnNames) {
		for (String col:columnNames) addColumn(col);
		vals=new Vector<>();
	}
	
	public void addRow(V value,Object...display) {
		synchronized (vals) {
			vals.add(value);
			super.addRow(display);
		}
	}
	
	public void addRow(V value,Vector<Object> display) {
		synchronized (vals) {
			vals.add(value);
			super.addRow(display);
		}
	}
	
	
	public void setData(V value,Object...display) {
		synchronized (vals) {
			int row=vals.indexOf(value);
			if (row>-1) {
				for (int i=0;i<display.length;i++) {
					if (display[i]!=null)
						setValueAt(display[i], row, i);
				}
			}
		}
	}
	
	public void removeRowByValue(V value) {
		synchronized (vals) {
			int row=vals.indexOf(value);
			if (row>-1) {
				removeRow(row);
			}
		}
	}
	
	@Override
	public void removeRow(int row) {
		synchronized (vals) {
			vals.remove(row);
			super.removeRow(row);
		}
	}
	
	public V getValue(int row) {
		synchronized (vals) {
			return vals.get(row);
		}
	}
	
	@Override
	public Object getValueAt(int row, int column) {
		if (row<getRowCount())
			return super.getValueAt(row, column);
		else
			return null;
	}
	
	@Override
	public void setRowCount(int rowCount) {
		synchronized (vals) {
			vals.setSize(rowCount);
			super.setRowCount(rowCount);
		}
	}
}
