package settings;

import java.nio.file.Path;
import java.util.LinkedHashMap;
import java.util.Locale;
import java.util.Map;
import java.util.Observable;
import java.util.Set;

public class Settings2 extends Observable implements Settings{

	private Map<String, SettingVal> settings=new LinkedHashMap<>();
	
	
	@Override
	public void fireSettingsChanged(String name) {
		setChanged();
		notifyObservers(name);
	}

	@Override
	public void set(String name, String value) {
		settings.put(name, new StringSetting(value));
	}

	@Override
	public void set(String name, int value) {
		settings.put(name, new IntSetting(value));
	}

	@Override
	public void set(String name, Locale l) {
		settings.put(name, new LocaleSetting(l));
	}

	@Override
	public void set(String name, String[] values) {
		settings.put(name, new StringArraySetting(values));
		
	}

	@Override
	public void set(String name, int[] values) {
		settings.put(name, new IntArraySetting(values));
	}

	@Override
	public void set(String name, boolean value) {
		settings.put(name, new BooleanSetting(value));
	}

	@Override
	public void set(String name, Path value) {
		if (value!=null){
			settings.put(name, new PathSetting(value));
		}
	}

	@Override
	public String getString(String name) {
		return ((StringSetting)settings.get(name)).val;
	}

	@Override
	public int[] getIntArray(String name) {
		return ((IntArraySetting)settings.get(name)).val;
	}

	@Override
	public String[] getStringArray(String name) {
		return ((StringArraySetting)settings.get(name)).val;
	}

	@Override
	public Integer getInt(String name) {
		return ((IntSetting)settings.get(name)).val;
	}

	@Override
	public boolean getBoolean(String name) {
		return ((BooleanSetting)settings.get(name)).val;
	}

	@Override
	public Double getDouble(String name) {
		return ((DoubleSetting)settings.get(name)).val;
	}

	@Override
	public Locale getLocale(String name) {
		return ((LocaleSetting)settings.get(name)).val;
	}


	@Override
	public Path getPath(String name) {
		return ((PathSetting)settings.get(name)).val;
	}

	private interface SettingVal{
		@Override
		public String toString();
	}
	
	private class IntSetting implements SettingVal{
		private int val;
		
		IntSetting(int val) {
			this.val=val;
		}
		
		@Override
		public String toString() {
			return Integer.toString(val);
		}
	}
	
	private class DoubleSetting implements SettingVal{
		private double val;
		
		DoubleSetting(double val) {
			this.val=val;
		}
		
		@Override
		public String toString() {
			return Double.toString(val);
		}
	}
	private class PathSetting implements SettingVal{
		private Path val;
		
		PathSetting(Path val) {
			this.val=val;
		}
		
		@Override
		public String toString() {
			return val.toString();
		}
	}
	private class StringSetting implements SettingVal{
		private String val;
		
		StringSetting(String val) {
			this.val=val;
		}
		
		@Override
		public String toString() {
			return val;
		}
	}

	private class BooleanSetting implements SettingVal{
		private boolean val;
		
		BooleanSetting(boolean val) {
			this.val=val;
		}
		
		@Override
		public String toString() {
			return val?"1":"0";
		}
	}
	
	private class LocaleSetting implements SettingVal{
		private Locale val;
		
		LocaleSetting(Locale val) {
			this.val=val;
		}
		
		@Override
		public String toString() {
			return val.getLanguage();
		}
	}
	
	private class StringArraySetting implements SettingVal{
		private String[] val;
		
		StringArraySetting(String[] val) {
			this.val=val;
		}
		
		@Override
		public String toString() {
			if (val.length>0) {
				StringBuilder sb=new StringBuilder();

				for (int i=0;i<val.length-1;i++) {
					sb.append(val[i]);
					sb.append(';');
				}
				sb.append(val[val.length-1]);
				return sb.toString();
			} else{
				return null;
			}
		}
	}
	
	private class IntArraySetting implements SettingVal{
		private int[] val;
		
		IntArraySetting(int[] val) {
			this.val=val;
		}
		
		@Override
		public String toString() {
			if (val.length>0) {
				StringBuilder sb=new StringBuilder();

				for (int i=0;i<val.length-1;i++) {
					sb.append(val[i]);
					sb.append(';');
				}
				sb.append(val[val.length-1]);
				return sb.toString();
			} else{
				return null;
			}
		}
	}
	
	
	@Override
	public void set(String name, double value) {
		settings.put(name, new DoubleSetting(value));
	}

	@Override
	public Set<String> getSettingsKeyset() {
		return settings.keySet();
	}

	@Override
	public String toString(String name) {
		return settings.get(name).toString();
	}

}
